with Ada.Text_IO; use Ada.Text_IO;
with Ada.Sequential_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Exceptions; use Ada.Exceptions;

procedure BIN2ASM is
  type Int8 is mod 2 ** 8;
  package Binary_IO is new Ada.Sequential_IO(Int8);
  package Text_Mod_IO is new Modular_IO(Int8);

  Program_Description : String := "BIN2ASM Ada Converter v1.01 for Win32 console";
  Asm_File            : Ada.Text_IO.File_type;
  Asm_File_Name       : Unbounded_String;
  Bin_File            : Binary_IO.File_type;
  Label_Name          : Unbounded_String;
  Byte                : Int8;
  Index_Dot           : Integer;
  DB_String           : String := " db ";
begin
  if Argument_Count = 0 then
    Put_Line(Program_Description);
    Put_Line("BIN2ASM binary_filename [asm_filename] [label]");
  else
    Binary_IO.Open (File => Bin_File, Name => Argument(1), Mode => Binary_IO.In_File);
      if Argument_Count < 2 then
        Asm_File_Name := To_Unbounded_String(Argument(1));
        Append(Asm_File_Name, ".asm");
      else
        Asm_File_Name := To_Unbounded_String(Argument(2));
      end if;
    Ada.Text_IO.Create (File => Asm_File, Name => To_String(Asm_File_Name), Mode => Ada.Text_IO.Out_File);
      if Argument_Count < 3 then
        Index_Dot := (Index (Asm_File_Name, "."));
          if Index_Dot > 1 then --       ?   ,    .
            Label_Name := To_Unbounded_String(Slice(Asm_File_Name,1,Index_Dot - 1));
          end if;
      else
        Label_Name := To_Unbounded_String(Argument(3));
      end if;
    Put_Line(Asm_File,"; " & Program_Description);
    New_Line(Asm_File);
    Put(Asm_File,To_String(Label_Name) & DB_String);

      while not Binary_IO.End_Of_File(Bin_File) loop
        for I in 1 .. 18 loop
          Binary_IO.Read(Bin_File,Byte);
          Text_Mod_IO.Put(Asm_File,Byte,Width => 1);
            if Binary_IO.End_Of_File(Bin_File) then
              exit;
            end if;
            if I /= 18 then
              Put(Asm_File,",");
            else
              New_Line(Asm_File);
              Put(Asm_File,DB_String);
            end if;
        end loop;
      end loop;

    Binary_IO.Close(Bin_File);
    Ada.Text_IO.Close(Asm_File);
    Put_Line("Done!");
  end if;

  exception
    when others =>
      Put_Line("Error! Wrong file name?");

end;

 ---------------------------------------
 -- E-mail: demon.adramelek@gmail.com --
 ---------------------------------------